@@ -10,22 +10,33 @@ from subprocess import DEVNULL, PIPE, CalledProcessError, run
 from tempfile import TemporaryDirectory
 from typing import Optional, cast
 
+import deprecation
 from packaging.version import Version
 from PIL import Image
 
 import pikepdf
 
 
+@deprecation.deprecated(
+    deprecated_in="5.1.5", removed_in="6.0", details="Use extract_jbig2_bytes instead"
+)
 def extract_jbig2(
     im_obj: pikepdf.Object, globals_obj: Optional[pikepdf.Object] = None
-) -> Image.Image:
+) -> Image.Image:  # pragma: no cover
 
-    with TemporaryDirectory(prefix='pikepdf', suffix='.jbig2') as tmpdir:
+    with TemporaryDirectory(prefix='pikepdf-', suffix='.jbig2') as tmpdir:
         image_path = Path(tmpdir) / "image"
         global_path = Path(tmpdir) / "global"
         output_path = Path(tmpdir) / "outfile"
 
-        args = ["jbig2dec", "-e", "-o", os.fspath(output_path)]
+        args = [
+            "jbig2dec",
+            "--embedded",
+            "--format",
+            "png",
+            "--output",
+            os.fspath(output_path),
+        ]
 
         # Get the raw stream, because we can't decode im_obj - that is why we are here
         # (Strictly speaking we should remove any non-JBIG2 filters if double encoded)
@@ -45,6 +56,36 @@ def extract_jbig2(
         return im
 
 
+def extract_jbig2_bytes(jbig2: bytes, jbig2_globals: bytes) -> bytes:
+    with TemporaryDirectory(prefix='pikepdf-', suffix='.jbig2') as tmpdir:
+        image_path = Path(tmpdir) / "image"
+        global_path = Path(tmpdir) / "global"
+        output_path = Path(tmpdir) / "outfile"
+
+        args = [
+            "jbig2dec",
+            "--embedded",
+            "--format",
+            "png",
+            "--output",
+            os.fspath(output_path),
+        ]
+
+        # Get the raw stream, because we can't decode im_obj - that is why we are here
+        # (Strictly speaking we should remove any non-JBIG2 filters if double encoded)
+        image_path.write_bytes(jbig2)
+
+        if len(jbig2_globals) > 0:
+            global_path.write_bytes(jbig2_globals)
+            args.append(os.fspath(global_path))
+
+        args.append(os.fspath(image_path))
+
+        run(args, stdout=DEVNULL, check=True)
+        with Image.open(output_path) as im:
+            return im.tobytes()
+
+
 def jbig2dec_available() -> bool:
     try:
         proc = run(['jbig2dec', '--version'], stdout=PIPE, check=True, encoding='ascii')
