@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2017, James R. Barlow (https://github.com/jbarlow83/)
 
-import struct
+
 from abc import ABC, abstractmethod
 from decimal import Decimal
 from io import BytesIO
@@ -15,18 +15,16 @@ from typing import (
     Any,
     BinaryIO,
     Callable,
-    Dict,
-    List,
     NamedTuple,
     Optional,
     Tuple,
     Type,
     TypeVar,
     Union,
+    cast,
 )
 
 from PIL import Image, ImageCms
-from PIL.TiffTags import TAGS_V2 as TIFF_TAGS
 
 from pikepdf import (
     Array,
@@ -40,6 +38,8 @@ from pikepdf import (
     String,
     jbig2,
 )
+from pikepdf._qpdf import Buffer
+from pikepdf.models import _transcoding
 
 
 class DependencyError(Exception):
@@ -68,14 +68,13 @@ def array_str(value: Union[Object, str, list]):
     def _array_str(item):
         if isinstance(item, (list, Array)):
             return [_array_str(subitem) for subitem in item]
-        elif isinstance(item, (Stream, Dictionary, bytes, int)):
+        if isinstance(item, (Stream, Dictionary, bytes, int)):
             return item
-        elif isinstance(item, (Name, str)):
+        if isinstance(item, (Name, str)):
             return str(item)
-        elif isinstance(item, (String)):
+        if isinstance(item, (String)):
             return bytes(item)
-        else:
-            raise NotImplementedError(value)
+        raise NotImplementedError(value)
 
     result = _array_str(value)
     if not isinstance(result, list):
@@ -89,7 +88,7 @@ def dict_or_array_dict(value):
     if isinstance(value, Dictionary):
         return [value.as_dict()]
     if isinstance(value, Array):
-        return [v for v in value.as_list()]
+        return list(value.as_list())
     raise NotImplementedError(value)
 
 
@@ -268,22 +267,21 @@ class PdfImageBase(ABC):
             m = 'Separation'
         elif self.indexed:
             m = 'P'
-        elif self.bits_per_component == 1:
+        elif self.colorspace == '/DeviceGray' and self.bits_per_component == 1:
             m = '1'
-        elif 2 <= self.bits_per_component <= 8:
-            if self.colorspace == '/DeviceRGB':
-                m = 'RGB'
-            elif self.colorspace == '/DeviceGray':
-                m = 'L'
-            elif self.colorspace == '/DeviceCMYK':
-                m = 'CMYK'
-            elif self.colorspace == '/ICCBased':
-                try:
-                    m = self._approx_mode_from_icc()
-                except (ValueError, TypeError) as e:
-                    raise NotImplementedError(
-                        "Not sure how to handle PDF image of this type"
-                    ) from e
+        elif self.colorspace == '/DeviceGray' and self.bits_per_component > 1:
+            m = 'L'
+        elif self.colorspace == '/DeviceRGB':
+            m = 'RGB'
+        elif self.colorspace == '/DeviceCMYK':
+            m = 'CMYK'
+        elif self.colorspace == '/ICCBased':
+            try:
+                m = self._approx_mode_from_icc()
+            except (ValueError, TypeError) as e:
+                raise NotImplementedError(
+                    "Not sure how to handle PDF image of this type"
+                ) from e
         if m == '':
             raise NotImplementedError(
                 "Not sure how to handle PDF image of this type"
@@ -524,7 +522,7 @@ class PdfImage(PdfImageBase):
             stream.write(self._generate_ccitt_header(data, icc=icc))
             stream.write(data)
             return '.tif'
-        elif filters == ['/DCTDecode'] and (
+        if filters == ['/DCTDecode'] and (
             self.mode == 'L' or normal_dct_rgb() or normal_dct_cmyk()
         ):
             stream.write(data)
@@ -532,19 +530,64 @@ class PdfImage(PdfImageBase):
 
         raise NotExtractableError()
 
-    def _extract_transcoded(self) -> Image.Image:
-        # Reminder Pillow palette byte order unintentionally changed in 8.3.0
-        # https://github.com/python-pillow/Pillow/issues/5595
-        # 8.2.0: all aligned by channel (very nonstandard)
-        # 8.3.0: all channels for one color followed by the next color (e.g. RGBRGBRGB)
+    def _extract_transcoded_1248bits(self) -> Image.Image:
+        """Extract an image when there are 1/2/4/8 bits packed in byte data."""
 
-        im = None
+        stride = 0  # tell Pillow to calculate stride from line width
+        scale = 0 if self.mode == 'L' else 1
+        if self.bits_per_component in (2, 4):
+            buffer, stride = _transcoding.unpack_subbyte_pixels(
+                self.read_bytes(), self.size, self.bits_per_component, scale
+            )
+        elif self.bits_per_component == 8:
+            buffer = cast(memoryview, self.get_stream_buffer())
+        else:
+            raise InvalidPdfImageError("BitsPerComponent must be 1, 2, 4, 8, or 16")
+
+        if self.mode == 'P' and self.palette is not None:
+            base_mode, palette = self.palette
+            im = _transcoding.image_from_buffer_and_palette(
+                buffer,
+                self.size,
+                stride,
+                base_mode,
+                palette,
+            )
+        else:
+            im = _transcoding.image_from_byte_buffer(buffer, self.size, stride)
+        return im
+
+    def _extract_transcoded_1bit(self) -> Image.Image:
+        if self.mode in ('RGB', 'CMYK'):
+            raise UnsupportedImageTypeError("1-bit RGB and CMYK are not supported")
+        try:
+            data = self.read_bytes()
+        except (RuntimeError, PdfError) as e:
+            if (
+                'read_bytes called on unfilterable stream' in str(e)
+                and not jbig2.jbig2dec_available()
+            ):
+                raise DependencyError(
+                    "jbig2dec - not installed or installed version is too old "
+                    "(older than version 0.15)"
+                ) from None
+            raise
+
+        im = Image.frombytes('1', self.size, data)
+
+        if self.palette is not None:
+            base_mode, palette = self.palette
+            im = _transcoding.fix_1bit_palette_image(im, base_mode, palette)
+
+        return im
+
+    def _extract_transcoded(self) -> Image.Image:
         if self.mode in {'DeviceN', 'Separation'}:
             raise HifiPrintImageNotTranscodableError()
 
         if self.mode == 'RGB' and self.bits_per_component == 8:
-            # No point in accessing the buffer here, size qpdf decodes to 3-byte
-            # RGB and Pillow needs RGBX for raw access
+            # Cannot use the zero-copy .get_stream_buffer here, we have 3-byte
+            # RGB and Pillow needs RGBX.
             im = Image.frombuffer(
                 'RGB', self.size, self.read_bytes(), 'raw', 'RGB', 0, 1
             )
@@ -552,81 +595,14 @@ class PdfImage(PdfImageBase):
             im = Image.frombuffer(
                 'CMYK', self.size, self.get_stream_buffer(), 'raw', 'CMYK', 0, 1
             )
-        elif self.mode in ('L', 'P') and 2 <= self.bits_per_component <= 8:
-            buffer = self.get_stream_buffer()
-            stride = 0  # tell Pillow to calculate stride from line width
-            ystep = 1  # image is top to bottom in memory
-            im = Image.frombuffer('L', self.size, buffer, "raw", 'L', stride, ystep)
-
-            shift = 8 - self.bits_per_component
-            if self.mode == 'L' and self.bits_per_component < 8:
-                graybytes = im.tobytes()
-                output = bytearray(len(graybytes))
-                for n, val in enumerate(graybytes):
-                    output[n] = val << shift
-                im = Image.frombytes('L', self.size, bytes(output))
-            elif self.mode == 'P' and self.palette is not None:
-                base_mode, palette = self.palette
-                if base_mode == 'RGB':
-                    im.putpalette(palette, rawmode=base_mode)
-                elif base_mode == 'L':
-                    # Pillow does not fully support palettes with rawmode='L'.
-                    # Convert to RGB palette.
-                    gray_palette = palette
-                    palette = b''
-                    for entry in gray_palette:
-                        palette += bytes([entry << shift]) * 3
-                    im.putpalette(palette, rawmode='RGB')
-                elif base_mode == 'CMYK':
-                    # Pillow does not support CMYK with palettes; convert manually
-                    with memoryview(buffer) as mv:
-                        output = bytearray(4 * len(mv))
-                        for n, pal_idx in enumerate(mv):
-                            output[4 * n : 4 * (n + 1)] = palette[
-                                4 * pal_idx : 4 * (pal_idx + 1)
-                            ]
-                    im = Image.frombuffer(
-                        'CMYK', self.size, data=output, decoder_name='raw'
-                    )
-                else:
-                    raise NotImplementedError(f'palette with {base_mode}')
+        # elif self.mode == '1':
         elif self.bits_per_component == 1:
-            if self.filters and self.filters[0] == '/JBIG2Decode':
-                if not jbig2.jbig2dec_available():
-                    raise DependencyError(
-                        "jbig2dec - not installed or installed version is too old "
-                        "(older than version 0.15)"
-                    )
-                jbig2_globals_obj = self.filter_decodeparms[0][1].get('/JBIG2Globals')
-                im = jbig2.extract_jbig2(self.obj, jbig2_globals_obj)
-            else:
-                data = self.read_bytes()
-                im = Image.frombytes('1', self.size, data)
+            im = self._extract_transcoded_1bit()
+        elif self.mode in ('L', 'P') and self.bits_per_component <= 8:
+            im = self._extract_transcoded_1248bits()
         else:
             raise UnsupportedImageTypeError(repr(self) + ", " + repr(self.obj))
 
-        if (
-            self.mode == 'P'
-            and self.bits_per_component == 1
-            and self.palette is not None
-        ):
-            # Fix paletted 1-bit images
-            base_mode, palette = self.palette
-            if base_mode == 'RGB' and palette != b'\x00\x00\x00\xff\xff\xff':
-                im = im.convert('P')
-                im.putpalette(palette, rawmode=base_mode)
-                gp = im.getpalette()
-                if gp:
-                    gp[765:768] = gp[3:6]  # work around Pillow bug
-                    im.putpalette(gp)
-            elif base_mode == 'L' and palette != b'\x00\xff':
-                im = im.convert('P')
-                im.putpalette(palette, rawmode=base_mode)
-                gp = im.getpalette()
-                if gp:
-                    gp[255] = gp[1]  # work around Pillow bug
-                    im.putpalette(gp)
-
         if self.colorspace == '/ICCBased' and self.icc is not None:
             im.info['icc_profile'] = self.icc.tobytes()
 
@@ -658,7 +634,7 @@ class PdfImage(PdfImageBase):
             if im.mode == 'CMYK':
                 im.save(stream, format='tiff', compression='tiff_adobe_deflate')
                 return '.tiff'
-            elif im:
+            if im:
                 im.save(stream, format='png')
                 return '.png'
         except PdfError as e:
@@ -728,7 +704,7 @@ class PdfImage(PdfImageBase):
 
     def get_stream_buffer(
         self, decode_level: StreamDecodeLevel = StreamDecodeLevel.specialized
-    ):
+    ) -> Buffer:
         """Access this image with the buffer protocol."""
         return self.obj.get_stream_buffer(decode_level=decode_level)
 
@@ -779,69 +755,14 @@ class PdfImage(PdfImageBase):
         photometry = 1 if black_is_one else 0
 
         img_size = len(data)
-        tiff_header_struct = '<' + '2s' + 'H' + 'L' + 'H'
-
-        tag_keys = {tag.name: key for key, tag in TIFF_TAGS.items()}  # type: ignore
-        ifd_struct = '<HHLL'
 
         if icc is None:
             icc = b''
 
-        class IFD(NamedTuple):
-            key: int
-            typecode: Any
-            count_: int
-            data: Union[int, Callable[[], Optional[int]]]
-
-        ifds: List[IFD] = []
-
-        def header_length(ifd_count) -> int:
-            return (
-                struct.calcsize(tiff_header_struct)
-                + struct.calcsize(ifd_struct) * ifd_count
-                + 4
-            )
-
-        def add_ifd(
-            tag_name: str, data: Union[int, Callable[[], Optional[int]]], count: int = 1
-        ):
-            key = tag_keys[tag_name]
-            typecode = TIFF_TAGS[key].type  # type: ignore
-            ifds.append(IFD(key, typecode, count, data))
-
-        image_offset = None
-        add_ifd('ImageWidth', self.width)
-        add_ifd('ImageLength', self.height)
-        add_ifd('BitsPerSample', 1)
-        add_ifd('Compression', ccitt_group)
-        add_ifd('PhotometricInterpretation', int(photometry))
-        add_ifd('StripOffsets', lambda: image_offset)
-        add_ifd('RowsPerStrip', self.height)
-        add_ifd('StripByteCounts', img_size)
-
-        icc_offset = 0
-        if icc:
-            add_ifd('ICCProfile', lambda: icc_offset, count=len(icc))
-
-        icc_offset = header_length(len(ifds))
-        image_offset = icc_offset + len(icc)
-
-        ifd_args = [(arg() if callable(arg) else arg) for ifd in ifds for arg in ifd]
-        tiff_header = struct.pack(
-            (tiff_header_struct + ifd_struct[1:] * len(ifds) + 'L'),
-            b'II',  # Byte order indication: Little endian
-            42,  # Version number (always 42)
-            8,  # Offset to first IFD
-            len(ifds),  # Number of tags in IFD
-            *ifd_args,
-            0,  # Last IFD
+        return _transcoding.generate_ccitt_header(
+            self.size, img_size, ccitt_group, photometry, icc
         )
 
-        if icc:
-            tiff_header += icc
-
-        return tiff_header
-
     def show(self):  # pragma: no cover
         """Show the image however PIL wants to."""
         self.as_pil_image().show()
@@ -892,7 +813,7 @@ class PdfJpxImage(PdfImage):
             return super_colorspaces
         if self._jpxpil.mode == 'L':
             return ['/DeviceGray']
-        elif self._jpxpil.mode == 'RGB':
+        if self._jpxpil.mode == 'RGB':
             return ['/DeviceRGB']
         raise NotImplementedError('Complex JP2 colorspace')
 
@@ -986,14 +907,12 @@ class PdfInlineImage(PdfImageBase):
                 name = obj.unparse(resolved=True)
                 assert isinstance(name, bytes)
                 return remap_names.get(name, name)
-            else:
-                return obj.unparse(resolved=True)
-        elif isinstance(obj, bool):
+            return obj.unparse(resolved=True)
+        if isinstance(obj, bool):
             return b'true' if obj else b'false'  # Lower case for PDF spec
-        elif isinstance(obj, (int, Decimal, float)):
+        if isinstance(obj, (int, Decimal, float)):
             return str(obj).encode('ascii')
-        else:
-            raise NotImplementedError(repr(obj))
+        raise NotImplementedError(repr(obj))
 
     def _metadata(self, name, type_, default):
         return metadata_from_obj(self.obj, name, type_, default)
