@@ -20,6 +20,7 @@ from typing import (
     NamedTuple,
     Optional,
     Set,
+    Tuple,
     Type,
     Union,
 )
@@ -52,7 +53,7 @@ XMP_NS_XMP = "http://ns.adobe.com/xap/1.0/"
 XMP_NS_XMP_MM = "http://ns.adobe.com/xap/1.0/mm/"
 XMP_NS_XMP_RIGHTS = "http://ns.adobe.com/xap/1.0/rights/"
 
-DEFAULT_NAMESPACES = [
+DEFAULT_NAMESPACES: List[Tuple[str, str]] = [
     ('adobe:ns:meta/', 'x'),
     (XMP_NS_DC, 'dc'),
     (XMP_NS_PDF, 'pdf'),
@@ -243,41 +244,9 @@ class AuthorConverter(Converter):
     def docinfo_from_xmp(xmp_val):
         if isinstance(xmp_val, str):
             return xmp_val
-        elif xmp_val is None or xmp_val == [None]:
+        if xmp_val is None or xmp_val == [None]:
             return None
-        else:
-            return '; '.join(xmp_val)
-
-
-def _fromisoformat_py36(date_string: str) -> datetime:
-    """Backported equivalent of datetime.fromisoformat
-
-    Can remove whenever we drop Python 3.6 support.
-    """
-    # strptime %z can't parse a timezone with punctuation
-    if re.search(r'[+-]\d{2}[-:]\d{2}$', date_string):
-        date_string = date_string[:-3] + date_string[-2:]
-    formats = [
-        "%Y-%m-%dT%H:%M:%S%z",
-        "%Y-%m-%dT%H:%M:%S",
-        "%Y-%m-%dT%H:%M:%S.%f%z",
-        "%Y-%m-%dT%H:%M:%S.%f",
-    ]
-    for fmt in formats:
-        try:
-            return datetime.strptime(date_string, fmt)
-        except ValueError:
-            continue
-    raise ValueError(
-        f"Could not parse ISO date: {date_string}. Try Python 3.7+, which "
-        "has better support for ISO dates."
-    )
-
-
-if sys.version_info < (3, 7) or sys.implementation.name == 'pypy':
-    fromisoformat = _fromisoformat_py36
-else:
-    fromisoformat = datetime.fromisoformat
+        return '; '.join(xmp_val)
 
 
 class DateConverter(Converter):
@@ -291,7 +260,11 @@ class DateConverter(Converter):
     def docinfo_from_xmp(xmp_val):
         if xmp_val.endswith('Z'):
             xmp_val = xmp_val[:-1] + '+00:00'
-        dateobj = fromisoformat(xmp_val)
+        try:
+            dateobj = datetime.fromisoformat(xmp_val)
+        except IndexError:
+            # PyPy 3.7 may raise IndexError - convert to ValueError
+            raise ValueError(f"Invalid isoformat string: '{xmp_val}'") from None
         return encode_pdf_date(dateobj)
 
 
@@ -355,7 +328,7 @@ class PdfMetadata(MutableMapping):
     ]
 
     NS: Dict[str, str] = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
-    REVERSE_NS: Dict[str, str] = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
+    REVERSE_NS: Dict[str, str] = dict(DEFAULT_NAMESPACES)
 
     _PARSERS_OVERWRITE_INVALID_XML: Iterable[Callable[[bytes], Any]] = [
         _parser_basic,
@@ -399,8 +372,7 @@ class PdfMetadata(MutableMapping):
         def warn_or_raise(msg, e=None):
             if raise_failure:
                 raise ValueError(msg) from e
-            else:
-                warn(msg)
+            warn(msg)
 
         for uri, shortkey, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, shortkey)
@@ -452,7 +424,11 @@ class PdfMetadata(MutableMapping):
         for parser in parsers:
             try:
                 self._xmp = parser(data)
-            except (XMLSyntaxError if self.overwrite_invalid_xml else NeverRaise) as e:
+            except (
+                XMLSyntaxError
+                if self.overwrite_invalid_xml
+                else NeverRaise  # type: ignore
+            ) as e:
                 if str(e).startswith("Start tag expected, '<' not found") or str(
                     e
                 ).startswith("Document is empty"):
@@ -467,13 +443,16 @@ class PdfMetadata(MutableMapping):
                 for pi in pis:
                     etree.strip_tags(self._xmp, pi.tag)
                 self._get_rdf_root()
-            except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
+            except (
+                Exception  # pylint: disable=broad-except
+                if self.overwrite_invalid_xml
+                else NeverRaise
+            ) as e:
                 log.warning("Error occurred parsing XMP", exc_info=e)
                 self._xmp = _parser_replace_with_empty_xmp()
         else:
             log.warning("Error occurred parsing XMP")
             self._xmp = _parser_replace_with_empty_xmp()
-        return
 
     @ensure_loaded
     def __enter__(self):
@@ -579,7 +558,14 @@ class PdfMetadata(MutableMapping):
             return name
         if name.startswith('{'):
             return name
-        prefix, tag = name.split(':', maxsplit=1)
+        try:
+            prefix, tag = name.split(':', maxsplit=1)
+        except ValueError:
+            # If missing the namespace, put it in the top level namespace
+            # To do this completely correct we actually need to figure out
+            # the namespace based on context defined by parent tags. That
+            #   https://www.w3.org/2001/tag/doc/qnameids.html
+            prefix, tag = 'x', name
         uri = cls.NS[prefix]
         return str(QName(uri, tag))
 
